@@ -71,7 +71,12 @@ def AddSshTunnelArgs(parser, tunnel_through_iap_scope):
       '--tunnel-through-iap',
       action='store_true',
       help="""\
-      Tunnel the ssh connection through the Cloud Identity-Aware Proxy.
+      Tunnel the ssh connection through Cloud Identity-Aware Proxy for TCP
+      forwarding.
+
+      To learn more, see the
+      [IAP for TCP forwarding documentation](
+      https://cloud.google.com/iap/docs/tcp-forwarding-overview).
       """)
 
 
@@ -557,7 +562,8 @@ class IapTunnelProxyServerHelper(_BaseIapTunnelHelper):
     try:
       self._RunReceiveLocalData(conn, repr(socket_address))
     except EnvironmentError as e:
-      log.info('Socket error [%s] while receiving from client.', str(e))
+      log.info('Socket error [%s] while receiving from client.',
+               six.text_type(e))
     except:  # pylint: disable=bare-except
       log.exception('Error while receiving from client.')
 
