@@ -36,6 +36,22 @@ PREREQUISITE_OPTION_ERROR_MSG = """\
 Cannot specify --{opt} without --{prerequisite}.
 """
 
+DETAILED_HELP = {
+    'EXAMPLES':
+        """\
+          To create an environment called ``env-1'' with all the default values,
+          run:
+
+            $ {command} env-1
+
+          To create a new environment named ``env-1'' with the Google Compute
+          Engine machine-type ``n1-standard-8'', and the Google Compute Engine
+          network ``my-network'', run:
+
+            $ {command} env-1 --machine-type=n1-standard-8 --network=my-network
+        """
+}
+
 
 def _CommonArgs(parser):
   """Common arguments that apply to all ReleaseTracks."""
@@ -141,8 +157,8 @@ information on how to structure KEYs and VALUEs, run
       the created environment.""")
 
   image_version_type = arg_parsers.RegexpValidator(
-      r'^composer-(\d+\.\d+.\d+|latest)-airflow-(\d+\.\d+(?:\.\d+)?)',
-      'must be in the form \'composer-A.B.C-airflow-X.Y[.Z]\' or '
+      r'^composer-(\d+\.\d+.\d+(?:-[a-z]+\.\d+)?|latest)-airflow-(\d+\.\d+(?:\.\d+)?)',
+      'must be in the form \'composer-A.B.C[-D.E]-airflow-X.Y[.Z]\' or '
       '\'latest\' can be provided in place of the Cloud Composer version '
       'string. For example: \'composer-latest-airflow-1.10.0\'.')
   version_group.add_argument(
@@ -151,7 +167,8 @@ information on how to structure KEYs and VALUEs, run
       help="""Version of the image to run in the environment.
 
       The image version encapsulates the versions of both Cloud Composer
-      and Apache Airflow. Must be of the form `composer-A.B.C-airflow-X.Y[.Z]`.
+      and Apache Airflow. Must be of the form
+      `composer-A.B.C[-D.E]-airflow-X.Y[.Z]`.
 
       The Cloud Composer and Airflow versions are semantic versions.
       `latest` can be provided instead of an explicit Cloud Composer
@@ -160,6 +177,7 @@ information on how to structure KEYs and VALUEs, run
       portion, the patch version can be omitted and the current
       version will be selected. The version numbers that are used will
       be stored.""")
+  flags.AddIpAliasEnvironmentFlags(parser)
 
 
 @base.ReleaseTracks(base.ReleaseTrack.GA)
@@ -172,11 +190,21 @@ class Create(base.Command):
     {top_command} composer operations describe
   """
 
+  detailed_help = DETAILED_HELP
+  _support_web_server_cloud_sql_private_ip_ranges = True
+  _support_web_server_access_control = False
+
   @staticmethod
   def Args(parser):
     _CommonArgs(parser)
+    flags.AddPrivateIpEnvironmentFlags(parser, True)
 
   def Run(self, args):
+    self.ParseIpAliasConfigOptions(args)
+    self.ParsePrivateEnvironmentConfigOptions(args)
+    if self._support_web_server_cloud_sql_private_ip_ranges:
+      self.ParsePrivateEnvironmentWebServerCloudSqlRanges(args)
+
     flags.ValidateDiskSize('--disk-size', args.disk_size)
     self.env_ref = args.CONCEPTS.environment.Parse()
     env_name = self.env_ref.Name()
@@ -231,53 +259,8 @@ class Create(base.Command):
             'Error creating [{}]: {}'.format(self.env_ref.RelativeName(),
                                              six.text_type(e)))
 
-  def GetOperationMessage(self, args):
-    """Constructs Create message."""
-    return environments_api_util.Create(
-        self.env_ref,
-        args.node_count,
-        labels=args.labels,
-        location=self.zone,
-        machine_type=self.machine_type,
-        network=self.network,
-        subnetwork=self.subnetwork,
-        env_variables=args.env_variables,
-        airflow_config_overrides=args.airflow_configs,
-        service_account=args.service_account,
-        oauth_scopes=args.oauth_scopes,
-        tags=args.tags,
-        disk_size_gb=args.disk_size >> 30,
-        python_version=args.python_version,
-        image_version=self.image_version,
-        release_track=self.ReleaseTrack())
-
-
-@base.ReleaseTracks(base.ReleaseTrack.BETA)
-class CreateBeta(Create):
-  """Create and initialize a Cloud Composer environment.
-
-  If run asynchronously with `--async`, exits after printing an operation
-  that can be used to poll the status of the creation operation via:
-
-    {top_command} composer operations describe
-  """
-
-  @staticmethod
-  def Args(parser):
-    Create.Args(parser)
-    flags.AddPrivateIpAndIpAliasEnvironmentFlags(parser)
-
-  def Run(self, args):
-    self.ParseIpAliasConfigOptions(args)
-    self.ParsePrivateEnvironmentConfigOptions(args)
-    return super(CreateBeta, self).Run(args)
-
   def ParseIpAliasConfigOptions(self, args):
     """Parses the options for VPC-native configuration."""
-    if args.enable_private_environment and not args.enable_ip_alias:
-      raise command_util.InvalidUserInputError(
-          PREREQUISITE_OPTION_ERROR_MSG.format(
-              prerequisite='enable-ip-alias', opt='enable-private-environment'))
     if args.cluster_ipv4_cidr and not args.enable_ip_alias:
       raise command_util.InvalidUserInputError(
           PREREQUISITE_OPTION_ERROR_MSG.format(
@@ -299,6 +282,11 @@ class CreateBeta(Create):
 
   def ParsePrivateEnvironmentConfigOptions(self, args):
     """Parses the options for Private Environment configuration."""
+    if args.enable_private_environment and not args.enable_ip_alias:
+      raise command_util.InvalidUserInputError(
+          PREREQUISITE_OPTION_ERROR_MSG.format(
+              prerequisite='enable-ip-alias', opt='enable-private-environment'))
+
     if args.enable_private_endpoint and not args.enable_private_environment:
       raise command_util.InvalidUserInputError(
           PREREQUISITE_OPTION_ERROR_MSG.format(
@@ -311,6 +299,91 @@ class CreateBeta(Create):
               prerequisite='enable-private-environment',
               opt='master-ipv4-cidr'))
 
+  def ParsePrivateEnvironmentWebServerCloudSqlRanges(self, args):
+    if args.web_server_ipv4_cidr and not args.enable_private_environment:
+      raise command_util.InvalidUserInputError(
+          PREREQUISITE_OPTION_ERROR_MSG.format(
+              prerequisite='enable-private-environment',
+              opt='web-server-ipv4-cidr'))
+
+    if args.cloud_sql_ipv4_cidr and not args.enable_private_environment:
+      raise command_util.InvalidUserInputError(
+          PREREQUISITE_OPTION_ERROR_MSG.format(
+              prerequisite='enable-private-environment',
+              opt='cloud-sql-ipv4-cidr'))
+
+  def GetOperationMessage(self, args):
+    """Constructs Create message."""
+    return environments_api_util.Create(
+        self.env_ref,
+        args.node_count,
+        labels=args.labels,
+        location=self.zone,
+        machine_type=self.machine_type,
+        network=self.network,
+        subnetwork=self.subnetwork,
+        env_variables=args.env_variables,
+        airflow_config_overrides=args.airflow_configs,
+        service_account=args.service_account,
+        oauth_scopes=args.oauth_scopes,
+        tags=args.tags,
+        disk_size_gb=args.disk_size >> 30,
+        python_version=args.python_version,
+        image_version=self.image_version,
+        use_ip_aliases=args.enable_ip_alias,
+        cluster_secondary_range_name=args.cluster_secondary_range_name,
+        services_secondary_range_name=args.services_secondary_range_name,
+        cluster_ipv4_cidr_block=args.cluster_ipv4_cidr,
+        services_ipv4_cidr_block=args.services_ipv4_cidr,
+        private_environment=args.enable_private_environment,
+        private_endpoint=args.enable_private_endpoint,
+        master_ipv4_cidr=args.master_ipv4_cidr,
+        web_server_ipv4_cidr=args.web_server_ipv4_cidr,
+        cloud_sql_ipv4_cidr=args.cloud_sql_ipv4_cidr,
+        release_track=self.ReleaseTrack())
+
+
+@base.ReleaseTracks(base.ReleaseTrack.BETA)
+class CreateBeta(Create):
+  """Create and initialize a Cloud Composer environment.
+
+  If run asynchronously with `--async`, exits after printing an operation
+  that can be used to poll the status of the creation operation via:
+
+    {top_command} composer operations describe
+  """
+
+  _support_web_server_access_control = True
+
+  @staticmethod
+  def Args(parser):
+    Create.Args(parser)
+    web_server_group = parser.add_mutually_exclusive_group()
+    flags.WEB_SERVER_ALLOW_IP.AddToParser(web_server_group)
+    flags.WEB_SERVER_ALLOW_ALL.AddToParser(web_server_group)
+    flags.WEB_SERVER_DENY_ALL.AddToParser(web_server_group)
+
+  def Run(self, args):
+    if self._support_web_server_access_control:
+      self.ParseWebServerAccessControlConfigOptions(args)
+    return super(CreateBeta, self).Run(args)
+
+  def ParseWebServerAccessControlConfigOptions(self, args):
+    if (args.enable_private_environment and not args.web_server_allow_ip and
+        not args.web_server_allow_all and not args.web_server_deny_all):
+      raise command_util.InvalidUserInputError(
+          'Cannot specify --enable-private-environment without one of: ' +
+          '--web-server-allow-ip, --web-server-allow-all ' +
+          'or --web-server-deny-all')
+
+    # Default to allow all if no flag is specified.
+    self.web_server_access_control = (
+        environments_api_util.BuildWebServerAllowedIps(
+            args.web_server_allow_ip, args.web_server_allow_all or
+            not args.web_server_allow_ip, args.web_server_deny_all))
+    flags.ValidateIpRanges(
+        [acl['ip_range'] for acl in self.web_server_access_control])
+
   def GetOperationMessage(self, args):
     """See base class."""
     return environments_api_util.Create(
@@ -337,6 +410,9 @@ class CreateBeta(Create):
         private_environment=args.enable_private_environment,
         private_endpoint=args.enable_private_endpoint,
         master_ipv4_cidr=args.master_ipv4_cidr,
+        web_server_ipv4_cidr=args.web_server_ipv4_cidr,
+        cloud_sql_ipv4_cidr=args.cloud_sql_ipv4_cidr,
+        web_server_access_control=self.web_server_access_control,
         release_track=self.ReleaseTrack())
 
 
@@ -350,9 +426,15 @@ class CreateAlpha(CreateBeta):
     {top_command} composer operations describe
   """
 
+  _support_web_server_cloud_sql_private_ip_ranges = False
+  _support_web_server_access_control = False
+
   @staticmethod
   def Args(parser):
-    CreateBeta.Args(parser)
+    _CommonArgs(parser)
+
+    # Private IP falgs without ranges missing in alpha.
+    flags.AddPrivateIpEnvironmentFlags(parser, False)
 
     # Adding alpha arguments
     parser.add_argument(
