@@ -18,6 +18,8 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
+import ipaddress
+
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.api_lib.compute import constants
 from googlecloudsdk.api_lib.compute import forwarding_rules_utils as utils
@@ -27,19 +29,22 @@ from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.command_lib.compute import flags as compute_flags
 from googlecloudsdk.command_lib.compute.forwarding_rules import flags
 from googlecloudsdk.core import log
-import ipaddress
 import six
 from six.moves import range  # pylint: disable=redefined-builtin
 
 
 def _Args(parser, support_global_access, support_l7_internal_load_balancing,
-          support_mirroring_collector):
+          support_target_grpc_proxy, support_psc_google_apis,
+          support_all_protocol, support_target_service_attachment):
   """Add the flags to create a forwarding rule."""
 
   flags.AddUpdateArgs(
       parser,
-      include_l7_internal_load_balancing=support_l7_internal_load_balancing)
-  flags.AddIPProtocols(parser)
+      include_l7_internal_load_balancing=support_l7_internal_load_balancing,
+      include_target_grpc_proxy=support_target_grpc_proxy,
+      include_psc_google_apis=support_psc_google_apis,
+      include_target_service_attachment=support_target_service_attachment)
+  flags.AddIPProtocols(parser, support_all_protocol)
   flags.AddDescription(parser)
   flags.AddPortsAndPortRange(parser)
   flags.AddNetworkTier(
@@ -48,17 +53,19 @@ def _Args(parser, support_global_access, support_l7_internal_load_balancing,
   if support_global_access:
     flags.AddAllowGlobalAccess(parser)
 
-  if support_mirroring_collector:
-    flags.AddIsMirroringCollector(parser)
+  flags.AddIsMirroringCollector(parser)
 
   parser.add_argument(
       '--service-label',
       help='(Only for Internal Load Balancing): '
-           'https://cloud.google.com/compute/docs/load-balancing/internal/\n'
-           'The DNS label to use as the prefix of the fully qualified domain '
-           'name for this forwarding rule. The full name will be internally '
-           'generated and output as dnsName. If this field is not specified, '
-           'no DNS record will be generated and no DNS name will be output. ')
+      'https://cloud.google.com/load-balancing/docs/dns-names/\n'
+      'The DNS label to use as the prefix of the fully qualified domain '
+      'name for this forwarding rule. The full name will be internally '
+      'generated and output as dnsName. If this field is not specified, '
+      'no DNS record will be generated and no DNS name will be output. '
+      'You cannot use the `--service-label` flag  if the forwarding rule '
+      'references an internal IP address that has the '
+      '`--purpose=SHARED_LOADBALANCER_VIP` flag set.')
   flags.AddAddressesAndIPVersions(
       parser,
       required=False,
@@ -75,25 +82,32 @@ class CreateHelper(object):
   FORWARDING_RULE_ARG = None
 
   def __init__(self, holder, support_global_access,
-               support_l7_internal_load_balancing,
-               support_mirroring_collector):
+               support_l7_internal_load_balancing, support_target_grpc_proxy,
+               support_psc_google_apis, support_all_protocol,
+               support_target_service_attachment):
     self._holder = holder
     self._support_global_access = support_global_access
     self._support_l7_internal_load_balancing = support_l7_internal_load_balancing
-    self._support_mirroring_collector = support_mirroring_collector
+    self._support_target_grpc_proxy = support_target_grpc_proxy
+    self._support_psc_google_apis = support_psc_google_apis
+    self._support_all_protocol = support_all_protocol
+    self._support_target_service_attachment = support_target_service_attachment
 
   @classmethod
   def Args(cls, parser, support_global_access,
-           support_l7_internal_load_balancing,
-           support_mirroring_collector):
+           support_l7_internal_load_balancing, support_target_grpc_proxy,
+           support_psc_google_apis, support_all_protocol,
+           support_target_service_attachment):
     cls.FORWARDING_RULE_ARG = _Args(parser, support_global_access,
                                     support_l7_internal_load_balancing,
-                                    support_mirroring_collector)
+                                    support_target_grpc_proxy,
+                                    support_psc_google_apis,
+                                    support_all_protocol,
+                                    support_target_service_attachment)
 
   def ConstructProtocol(self, messages, args):
     if args.ip_protocol:
-      return messages.ForwardingRule.IPProtocolValueValuesEnum(
-          args.ip_protocol)
+      return messages.ForwardingRule.IPProtocolValueValuesEnum(args.ip_protocol)
     else:
       return
 
@@ -117,15 +131,59 @@ class CreateHelper(object):
 
   def _CreateGlobalRequests(self, client, resources, args, forwarding_rule_ref):
     """Create a globally scoped request."""
+
+    is_psc_google_apis = False
+    if hasattr(args,
+               'target_google_apis_bundle') and args.target_google_apis_bundle:
+      if not self._support_psc_google_apis:
+        raise exceptions.InvalidArgumentException(
+            '--target-google-apis-bundle',
+            'Private Service Connect for Google APIs (the target-google-apis-bundle option '
+            'for forwarding rules) is not supported in this API version.')
+      else:
+        is_psc_google_apis = True
+
     ports_all_specified, range_list = _ExtractPortsAndAll(args.ports)
     port_range = _ResolvePortRange(args.port_range, range_list)
-    if ports_all_specified:
+    load_balancing_scheme = _GetLoadBalancingScheme(args, client.messages,
+                                                    is_psc_google_apis)
+
+    if ports_all_specified and not is_psc_google_apis:
       raise exceptions.ToolException(
           '[--ports] can not be specified to all for global forwarding rules.')
-    if not port_range:
+    if not is_psc_google_apis and not port_range:
       raise exceptions.ToolException(
           '[--ports] is required for global forwarding rules.')
-    target_ref = utils.GetGlobalTarget(resources, args)
+
+    if is_psc_google_apis:
+      rule_name = forwarding_rule_ref.Name()
+      if len(rule_name) > 20 or rule_name[0].isdigit(
+      ) or not rule_name.isalnum():
+        raise exceptions.ToolException(
+            'A forwarding rule to Google APIs must have a name that is between '
+            ' 1-20 characters long, alphanumeric, starting with a letter.')
+
+      if port_range:
+        raise exceptions.ToolException(
+            '[--ports] is not allowed for PSC-GoogleApis forwarding rules.')
+      if load_balancing_scheme:
+        raise exceptions.InvalidArgumentException(
+            '--load-balancing-scheme',
+            'The --load-balancing-scheme flag is not allowed for PSC-GoogleApis'
+            ' forwarding rules.')
+
+      if args.target_google_apis_bundle in flags.PSC_GOOGLE_APIS_BUNDLES:
+        target_as_str = args.target_google_apis_bundle
+      else:
+        bundles_list = ', '.join(flags.PSC_GOOGLE_APIS_BUNDLES)
+        raise exceptions.InvalidArgumentException(
+            '--target-google-apis-bundle',
+            'The valid values for target-google-apis-bundle are: ' +
+            bundles_list)
+    else:
+      target_ref = utils.GetGlobalTarget(resources, args,
+                                         self._support_target_grpc_proxy)
+      target_as_str = target_ref.SelfLink()
     protocol = self.ConstructProtocol(client.messages, args)
 
     if args.address is None or args.ip_version:
@@ -137,17 +195,16 @@ class CreateHelper(object):
     address = self._ResolveAddress(resources, args,
                                    compute_flags.compute_scope.ScopeEnum.GLOBAL,
                                    forwarding_rule_ref)
-
     forwarding_rule = client.messages.ForwardingRule(
         description=args.description,
         name=forwarding_rule_ref.Name(),
         IPAddress=address,
         IPProtocol=protocol,
         portRange=port_range,
-        target=target_ref.SelfLink(),
+        target=target_as_str,
         ipVersion=ip_version,
         networkTier=_ConstructNetworkTier(client.messages, args),
-        loadBalancingScheme=_GetLoadBalancingScheme(args, client.messages))
+        loadBalancingScheme=load_balancing_scheme)
 
     if args.IsSpecified('network'):
       forwarding_rule.network = flags.NetworkArg(
@@ -158,21 +215,33 @@ class CreateHelper(object):
       forwarding_rule.allowGlobalAccess = args.allow_global_access
 
     request = client.messages.ComputeGlobalForwardingRulesInsertRequest(
-        forwardingRule=forwarding_rule,
-        project=forwarding_rule_ref.project)
+        forwardingRule=forwarding_rule, project=forwarding_rule_ref.project)
 
     return [(client.apitools_client.globalForwardingRules, 'Insert', request)]
 
   def _CreateRegionalRequests(self, client, resources, args,
                               forwarding_rule_ref):
     """Create a regionally scoped request."""
+    is_psc_ilb = False
+    if hasattr(args,
+               'target_service_attachment') and args.target_service_attachment:
+      if not self._support_target_service_attachment:
+        raise exceptions.InvalidArgumentException(
+            '--target-service-attachment',
+            'Private Service Connect for ILB (the target-service-attachment '
+            'option) is not supported in this API version.')
+      else:
+        is_psc_ilb = True
+
     target_ref, region_ref = utils.GetRegionalTarget(
         client,
         resources,
         args,
         forwarding_rule_ref,
         include_l7_internal_load_balancing=self
-        ._support_l7_internal_load_balancing)
+        ._support_l7_internal_load_balancing,
+        include_target_service_attachment=self
+        ._support_target_service_attachment)
     if not args.region and region_ref:
       args.region = region_ref
     protocol = self.ConstructProtocol(client.messages, args)
@@ -180,6 +249,13 @@ class CreateHelper(object):
     address = self._ResolveAddress(resources, args,
                                    compute_flags.compute_scope.ScopeEnum.REGION,
                                    forwarding_rule_ref)
+    load_balancing_scheme = _GetLoadBalancingScheme(args, client.messages,
+                                                    is_psc_ilb)
+    if is_psc_ilb and load_balancing_scheme:
+      raise exceptions.InvalidArgumentException(
+          '--load-balancing-scheme',
+          'The --load-balancing-scheme flag is not allowed for PSC-ILB '
+          'forwarding rules.')
 
     forwarding_rule = client.messages.ForwardingRule(
         description=args.description,
@@ -187,10 +263,16 @@ class CreateHelper(object):
         IPAddress=address,
         IPProtocol=protocol,
         networkTier=_ConstructNetworkTier(client.messages, args),
-        loadBalancingScheme=_GetLoadBalancingScheme(args, client.messages))
+        loadBalancingScheme=load_balancing_scheme)
 
     ports_all_specified, range_list = _ExtractPortsAndAll(args.ports)
-    if (target_ref.Collection() == 'compute.regionBackendServices') or (
+    if target_ref.Collection() == 'compute.serviceAttachments':
+      forwarding_rule.target = target_ref.SelfLink()
+      if args.network is not None:
+        forwarding_rule.network = flags.NetworkArg(
+            self._support_l7_internal_load_balancing).ResolveAsResource(
+                args, resources).SelfLink()
+    elif (target_ref.Collection() == 'compute.regionBackendServices') or (
         target_ref.Collection() == 'compute.targetInstances' and
         args.load_balancing_scheme == 'INTERNAL'):
       forwarding_rule.portRange = (
@@ -298,21 +380,29 @@ class CreateHelper(object):
 class Create(base.CreateCommand):
   """Create a forwarding rule to direct network traffic to a load balancer."""
 
-  _support_global_access = False
-  _support_l7_internal_load_balancing = False
-  _support_mirroring_collector = False
+  _support_global_access = True
+  _support_l7_internal_load_balancing = True
+  _support_target_grpc_proxy = True
+  _support_psc_google_apis = False
+  _support_all_protocol = False
+  _support_target_service_attachment = False
 
   @classmethod
   def Args(cls, parser):
     CreateHelper.Args(parser, cls._support_global_access,
                       cls._support_l7_internal_load_balancing,
-                      cls._support_mirroring_collector)
+                      cls._support_target_grpc_proxy,
+                      cls._support_psc_google_apis, cls._support_all_protocol,
+                      cls._support_target_service_attachment)
 
   def Run(self, args):
     holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
     return CreateHelper(holder, self._support_global_access,
                         self._support_l7_internal_load_balancing,
-                        self._support_mirroring_collector).Run(args)
+                        self._support_target_grpc_proxy,
+                        self._support_psc_google_apis,
+                        self._support_all_protocol,
+                        self._support_target_service_attachment).Run(args)
 
 
 @base.ReleaseTracks(base.ReleaseTrack.BETA)
@@ -320,7 +410,10 @@ class CreateBeta(Create):
   """Create a forwarding rule to direct network traffic to a load balancer."""
   _support_global_access = True
   _support_l7_internal_load_balancing = True
-  _support_mirroring_collector = True
+  _support_target_grpc_proxy = True
+  _support_psc_google_apis = False
+  _support_all_protocol = False
+  _support_target_service_attachment = False
 
 
 @base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@@ -328,7 +421,10 @@ class CreateAlpha(CreateBeta):
   """Create a forwarding rule to direct network traffic to a load balancer."""
   _support_global_access = True
   _support_l7_internal_load_balancing = True
-  _support_mirroring_collector = True
+  _support_target_grpc_proxy = True
+  _support_psc_google_apis = True
+  _support_all_protocol = True
+  _support_target_service_attachment = True
 
 
 Create.detailed_help = {
@@ -337,10 +433,11 @@ Create.detailed_help = {
 
 When creating a forwarding rule, exactly one of  ``--target-instance'',
 ``--target-pool'', ``--target-http-proxy'', ``--target-https-proxy'',
-``--target-ssl-proxy'', ``--target-tcp-proxy'', ``--target-vpn-gateway''
-or ``--backend-service'' must be specified."""
-                    .format(overview=flags.FORWARDING_RULES_OVERVIEW)),
-    'EXAMPLES': """
+``--target-grpc-proxy'', ``--target-ssl-proxy'', ``--target-tcp-proxy'',
+``--target-vpn-gateway'' or ``--backend-service'' must be specified.""".format(
+    overview=flags.FORWARDING_RULES_OVERVIEW)),
+    'EXAMPLES':
+        """
     To create a global forwarding rule that will forward all traffic on port
     8080 for IP address ADDRESS to a target http proxy PROXY, run:
 
@@ -354,7 +451,6 @@ or ``--backend-service'' must be specified."""
 """
 }
 
-
 CreateBeta.detailed_help = Create.detailed_help
 CreateAlpha.detailed_help = Create.detailed_help
 
@@ -384,8 +480,9 @@ def _ExtractPortsAndAll(ports_with_all):
 def _ResolvePortRange(port_range, port_range_list):
   """Reconciles deprecated port_range value and list of port ranges."""
   if port_range:
-    log.warning('The --port-range flag is deprecated. Use equivalent --ports=%s'
-                ' flag.', port_range)
+    log.warning(
+        'The --port-range flag is deprecated. Use equivalent --ports=%s'
+        ' flag.', port_range)
   elif port_range_list:
     port_range = _GetPortRange(port_range_list)
   return six.text_type(port_range) if port_range else None
@@ -398,15 +495,18 @@ def _GetPortList(range_list):
   return sorted(ports)
 
 
-def _GetLoadBalancingScheme(args, messages):
+def _GetLoadBalancingScheme(args, messages, is_psc):
   """Get load balancing scheme."""
+  if not args.load_balancing_scheme:
+    # The default is EXTERNAL for non-PSC forwarding rules.
+    return None if is_psc else messages.ForwardingRule.LoadBalancingSchemeValueValuesEnum.EXTERNAL
   if args.load_balancing_scheme == 'INTERNAL':
     return messages.ForwardingRule.LoadBalancingSchemeValueValuesEnum.INTERNAL
   elif args.load_balancing_scheme == 'EXTERNAL':
     return messages.ForwardingRule.LoadBalancingSchemeValueValuesEnum.EXTERNAL
   elif args.load_balancing_scheme == 'INTERNAL_SELF_MANAGED':
-    return (messages.ForwardingRule.LoadBalancingSchemeValueValuesEnum.
-            INTERNAL_SELF_MANAGED)
+    return (messages.ForwardingRule.LoadBalancingSchemeValueValuesEnum
+            .INTERNAL_SELF_MANAGED)
   elif args.load_balancing_scheme == 'INTERNAL_MANAGED':
     return (messages.ForwardingRule.LoadBalancingSchemeValueValuesEnum
             .INTERNAL_MANAGED)
