@@ -82,6 +82,9 @@ def _AddMutuallyExclusiveArgs(mutex_group, release_track):
                 api_adapter.CLOUDRUN: _ParseAddonDisabled,
                 api_adapter.APPLICATIONMANAGER: _ParseAddonDisabled,
                 api_adapter.CLOUDBUILD: _ParseAddonDisabled,
+                api_adapter.NODELOCALDNS: _ParseAddonDisabled,
+                api_adapter.GCEPDCSIDRIVER: _ParseAddonDisabled,
+                api_adapter.CONFIGCONNECTOR: _ParseAddonDisabled,
             }),
         dest='disable_addons',
         metavar='ADDON=ENABLED|DISABLED',
@@ -93,7 +96,10 @@ def _AddMutuallyExclusiveArgs(mutex_group, release_track):
 {application_manager}=ENABLED|DISABLED
 {network_policy}=ENABLED|DISABLED
 {cloudrun}=ENABLED|DISABLED
-{cloudbuild}=ENABLED|DISABLED""".format(
+{cloudbuild}=ENABLED|DISABLED
+{configconnector}=ENABLED|DISABLED
+{nodelocaldns}=ENABLED|DISABLED
+{gcepdcsidriver}=ENABLED|DISABLED""".format(
     hpa=api_adapter.HPA,
     ingress=api_adapter.INGRESS,
     dashboard=api_adapter.DASHBOARD,
@@ -102,6 +108,9 @@ def _AddMutuallyExclusiveArgs(mutex_group, release_track):
     application_manager=api_adapter.APPLICATIONMANAGER,
     cloudrun=api_adapter.CLOUDRUN,
     cloudbuild=api_adapter.CLOUDBUILD,
+    configconnector=api_adapter.CONFIGCONNECTOR,
+    nodelocaldns=api_adapter.NODELOCALDNS,
+    gcepdcsidriver=api_adapter.GCEPDCSIDRIVER,
     ))
 
   elif release_track == base.ReleaseTrack.BETA:
@@ -116,6 +125,9 @@ def _AddMutuallyExclusiveArgs(mutex_group, release_track):
                 api_adapter.ISTIO: _ParseAddonDisabled,
                 api_adapter.CLOUDRUN: _ParseAddonDisabled,
                 api_adapter.APPLICATIONMANAGER: _ParseAddonDisabled,
+                api_adapter.NODELOCALDNS: _ParseAddonDisabled,
+                api_adapter.GCEPDCSIDRIVER: _ParseAddonDisabled,
+                api_adapter.CONFIGCONNECTOR: _ParseAddonDisabled,
             }),
         dest='disable_addons',
         metavar='ADDON=ENABLED|DISABLED',
@@ -126,7 +138,10 @@ def _AddMutuallyExclusiveArgs(mutex_group, release_track):
 {istio}=ENABLED|DISABLED
 {application_manager}=ENABLED|DISABLED
 {network_policy}=ENABLED|DISABLED
-{cloudrun}=ENABLED|DISABLED""".format(
+{cloudrun}=ENABLED|DISABLED
+{configconnector}=ENABLED|DISABLED
+{nodelocaldns}=ENABLED|DISABLED
+{gcepdcsidriver}=ENABLED|DISABLED""".format(
     hpa=api_adapter.HPA,
     ingress=api_adapter.INGRESS,
     dashboard=api_adapter.DASHBOARD,
@@ -134,6 +149,9 @@ def _AddMutuallyExclusiveArgs(mutex_group, release_track):
     istio=api_adapter.ISTIO,
     application_manager=api_adapter.APPLICATIONMANAGER,
     cloudrun=api_adapter.CLOUDRUN,
+    configconnector=api_adapter.CONFIGCONNECTOR,
+    nodelocaldns=api_adapter.NODELOCALDNS,
+    gcepdcsidriver=api_adapter.GCEPDCSIDRIVER,
     ))
 
   else:
@@ -145,6 +163,8 @@ def _AddMutuallyExclusiveArgs(mutex_group, release_track):
                 api_adapter.HPA: _ParseAddonDisabled,
                 api_adapter.DASHBOARD: _ParseAddonDisabled,
                 api_adapter.NETWORK_POLICY: _ParseAddonDisabled,
+                api_adapter.CLOUDRUN: _ParseAddonDisabled,
+                api_adapter.NODELOCALDNS: _ParseAddonDisabled,
             }),
         dest='disable_addons',
         metavar='ADDON=ENABLED|DISABLED',
@@ -152,11 +172,15 @@ def _AddMutuallyExclusiveArgs(mutex_group, release_track):
 {hpa}=ENABLED|DISABLED
 {ingress}=ENABLED|DISABLED
 {dashboard}=ENABLED|DISABLED
-{network_policy}=ENABLED|DISABLED""".format(
+{network_policy}=ENABLED|DISABLED
+{cloudrun}=ENABLED|DISABLED
+{nodelocaldns}=ENABLED|DISABLED""".format(
     hpa=api_adapter.HPA,
     ingress=api_adapter.INGRESS,
     dashboard=api_adapter.DASHBOARD,
     network_policy=api_adapter.NETWORK_POLICY,
+    cloudrun=api_adapter.CLOUDRUN,
+    nodelocaldns=api_adapter.NODELOCALDNS,
     ))
 
   mutex_group.add_argument(
@@ -242,6 +266,7 @@ class Update(base.UpdateCommand):
     flags.AddStartCredentialRotationFlag(group)
     flags.AddCompleteIpRotationFlag(group)
     flags.AddCompleteCredentialRotationFlag(group)
+    flags.AddCloudRunConfigFlag(parser)
     flags.AddUpdateLabelsFlag(group)
     flags.AddRemoveLabelsFlag(group)
     flags.AddNetworkPolicyFlags(group)
@@ -252,23 +277,39 @@ class Update(base.UpdateCommand):
     flags.AddEnableBinAuthzFlag(group)
     flags.AddEnableStackdriverKubernetesFlag(group)
     flags.AddDailyMaintenanceWindowFlag(group, add_unset_text=True)
-    flags.AddRecurringMaintenanceWindowFlags(
-        group, hidden=True, is_update=True)
+    flags.AddRecurringMaintenanceWindowFlags(group, is_update=True)
     flags.AddResourceUsageExportFlags(group, is_update=True)
+    flags.AddReleaseChannelFlag(group, is_update=True, hidden=False)
+    flags.AddWorkloadIdentityFlags(group)
+    flags.AddWorkloadIdentityUpdateFlags(group)
     flags.AddDatabaseEncryptionFlag(group)
     flags.AddDisableDatabaseEncryptionFlag(group)
     flags.AddVerticalPodAutoscalingFlag(group)
-    flags.AddAutoprovisioningFlags(group)
+    flags.AddAutoprovisioningFlags(group, ga=True)
+    flags.AddEnableShieldedNodesFlags(group)
 
   def ParseUpdateOptions(self, args, locations):
     opts = container_command_util.ParseUpdateOptionsBase(args, locations)
     opts.resource_usage_bigquery_dataset = args.resource_usage_bigquery_dataset
     opts.clear_resource_usage_bigquery_dataset = \
         args.clear_resource_usage_bigquery_dataset
+    opts.cloud_run_config = args.cloud_run_config
     opts.enable_network_egress_metering = args.enable_network_egress_metering
     opts.enable_resource_consumption_metering = \
         args.enable_resource_consumption_metering
     opts.enable_intra_node_visibility = args.enable_intra_node_visibility
+    opts.enable_shielded_nodes = args.enable_shielded_nodes
+    opts.release_channel = args.release_channel
+    flags.ValidateCloudRunConfigUpdateArgs(args.cloud_run_config, args.disable_addons)
+    if args.disable_addons and api_adapter.NODELOCALDNS in args.disable_addons:
+      # NodeLocalDNS is being enabled or disabled
+      console_io.PromptContinue(
+          message='Enabling/Disabling NodeLocal DNSCache causes a re-creation '
+          'of all cluster nodes at versions 1.15 or above. '
+          'This operation is long-running and will block other '
+          'operations on the cluster (including delete) until it has run '
+          'to completion.',
+          cancel_on_no=True)
     return opts
 
   def Run(self, args):
@@ -483,7 +524,7 @@ to completion."""
 
     if not args.async_:
       adapter.WaitForOperation(
-          op_ref, 'Updating {0}'.format(cluster_ref.clusterId), timeout_s=1800)
+          op_ref, 'Updating {0}'.format(cluster_ref.clusterId), timeout_s=3600)
 
       log.UpdatedResource(cluster_ref)
       cluster_url = util.GenerateClusterUrl(cluster_ref)
@@ -524,6 +565,8 @@ class UpdateBeta(Update):
     flags.AddLoggingServiceFlag(group_logging_monitoring)
     flags.AddMonitoringServiceFlag(group_logging_monitoring)
     flags.AddEnableStackdriverKubernetesFlag(group)
+    flags.AddEnableLoggingMonitoringSystemOnlyFlag(group)
+    flags.AddEnableMasterSignalsFlags(group)
     flags.AddMasterAuthorizedNetworksFlags(
         parser, enable_group_for_update=group)
     flags.AddEnableLegacyAuthorizationFlag(group)
@@ -535,26 +578,35 @@ class UpdateBeta(Update):
     flags.AddRemoveLabelsFlag(group)
     flags.AddNetworkPolicyFlags(group)
     flags.AddDailyMaintenanceWindowFlag(group, add_unset_text=True)
-    flags.AddRecurringMaintenanceWindowFlags(
-        group, hidden=False, is_update=True)
+    flags.AddRecurringMaintenanceWindowFlags(group, is_update=True)
     flags.AddPodSecurityPolicyFlag(group)
     flags.AddEnableBinAuthzFlag(group)
     flags.AddAutoprovisioningFlags(group)
+    flags.AddAutoscalingProfilesFlag(group)
     flags.AddVerticalPodAutoscalingFlag(group)
     flags.AddResourceUsageExportFlags(group, is_update=True)
     flags.AddIstioConfigFlag(parser)
+    flags.AddCloudRunConfigFlag(parser)
     flags.AddEnableIntraNodeVisibilityFlag(group)
-    flags.AddWorkloadIdentityFlags(group)
+    flags.AddWorkloadIdentityFlags(group, use_workload_pool=False)
     flags.AddWorkloadIdentityUpdateFlags(group)
-    flags.AddEnableShieldedNodesFlags(group)
     flags.AddDatabaseEncryptionFlag(group)
     flags.AddDisableDatabaseEncryptionFlag(group)
-    flags.AddReleaseChannelFlag(group, is_update=True)
+    flags.AddReleaseChannelFlag(group, is_update=True, hidden=False)
+    flags.AddEnableShieldedNodesFlags(group)
+    flags.AddTpuFlags(group, enable_tpu_service_networking=True)
+    flags.AddMasterGlobalAccessFlag(group)
+    flags.AddEnableGvnicFlag(group)
+    flags.AddDisableDefaultSnatFlag(group, for_cluster_create=False)
+    flags.AddNotificationConfigFlag(group, hidden=True)
+    flags.AddPrivateIpv6GoogleAccessTypeFlag('v1beta1', group, hidden=True)
 
   def ParseUpdateOptions(self, args, locations):
+    flags.ValidateNotificationConfigFlag(args)
     opts = container_command_util.ParseUpdateOptionsBase(args, locations)
     opts.enable_pod_security_policy = args.enable_pod_security_policy
     opts.istio_config = args.istio_config
+    opts.cloud_run_config = args.cloud_run_config
     opts.resource_usage_bigquery_dataset = args.resource_usage_bigquery_dataset
     opts.enable_intra_node_visibility = args.enable_intra_node_visibility
     opts.clear_resource_usage_bigquery_dataset = \
@@ -562,14 +614,47 @@ class UpdateBeta(Update):
     opts.enable_network_egress_metering = args.enable_network_egress_metering
     opts.enable_resource_consumption_metering = args.enable_resource_consumption_metering
     flags.ValidateIstioConfigUpdateArgs(args.istio_config, args.disable_addons)
+    flags.ValidateCloudRunConfigUpdateArgs(args.cloud_run_config, args.disable_addons)
+    if args.disable_addons and api_adapter.NODELOCALDNS in args.disable_addons:
+      # NodeLocalDNS is being enabled or disabled
+      console_io.PromptContinue(
+          message='Enabling/Disabling NodeLocal DNSCache causes a re-creation '
+          'of all cluster nodes at versions 1.15 or above. '
+          'This operation is long-running and will block other '
+          'operations on the cluster (including delete) until it has run '
+          'to completion.',
+          cancel_on_no=True)
+    if args.disable_addons and api_adapter.GCEPDCSIDRIVER in args.disable_addons:
+      pdcsi_disabled = args.disable_addons[api_adapter.GCEPDCSIDRIVER]
+      if pdcsi_disabled:
+        # Persistent Disk CSI Driver is being disabled
+        console_io.PromptContinue(
+            message='If the Compute Engine Persistent Disk CSI Driver is '
+            'disabled, then any pods currently using PersistentVolumes owned '
+            'by the driver will fail to terminate. Any new pods that try to '
+            'use those PersistentVolumes will also fail to start.',
+            cancel_on_no=True)
+
     opts.enable_stackdriver_kubernetes = args.enable_stackdriver_kubernetes
+    opts.enable_logging_monitoring_system_only = args.enable_logging_monitoring_system_only
+    opts.master_logs = args.master_logs
+    opts.no_master_logs = args.no_master_logs
+    opts.enable_master_metrics = args.enable_master_metrics
     opts.release_channel = args.release_channel
+    opts.autoscaling_profile = args.autoscaling_profile
 
     # Top-level update options are automatically forced to be
     # mutually-exclusive, so we don't need special handling for these two.
     opts.identity_namespace = args.identity_namespace
-    opts.disable_workload_identity = args.disable_workload_identity
     opts.enable_shielded_nodes = args.enable_shielded_nodes
+    opts.enable_tpu = args.enable_tpu
+    opts.tpu_ipv4_cidr = args.tpu_ipv4_cidr
+    opts.enable_tpu_service_networking = args.enable_tpu_service_networking
+    opts.enable_master_global_access = args.enable_master_global_access
+    opts.enable_gvnic = args.enable_gvnic
+    opts.disable_default_snat = args.disable_default_snat
+    opts.notification_config = args.notification_config
+    opts.private_ipv6_google_access_type = args.private_ipv6_google_access_type
 
     return opts
 
@@ -591,6 +676,8 @@ class UpdateAlpha(Update):
     flags.AddLoggingServiceFlag(group_logging_monitoring)
     flags.AddMonitoringServiceFlag(group_logging_monitoring)
     flags.AddEnableStackdriverKubernetesFlag(group)
+    flags.AddEnableLoggingMonitoringSystemOnlyFlag(group)
+    flags.AddEnableMasterSignalsFlags(group)
     flags.AddMasterAuthorizedNetworksFlags(
         parser, enable_group_for_update=group)
     flags.AddEnableLegacyAuthorizationFlag(group)
@@ -602,28 +689,33 @@ class UpdateAlpha(Update):
     flags.AddRemoveLabelsFlag(group)
     flags.AddNetworkPolicyFlags(group)
     flags.AddAutoprovisioningFlags(group, hidden=False)
-    flags.AddAutoscalingProfilesFlag(group, hidden=True)
+    flags.AddAutoscalingProfilesFlag(group)
     flags.AddDailyMaintenanceWindowFlag(group, add_unset_text=True)
-    flags.AddRecurringMaintenanceWindowFlags(
-        group, hidden=False, is_update=True)
+    flags.AddRecurringMaintenanceWindowFlags(group, is_update=True)
     flags.AddPodSecurityPolicyFlag(group)
     flags.AddEnableBinAuthzFlag(group)
     flags.AddResourceUsageExportFlags(group, is_update=True)
     flags.AddVerticalPodAutoscalingFlag(group)
     flags.AddSecurityProfileForUpdateFlag(group)
     flags.AddIstioConfigFlag(parser)
+    flags.AddCloudRunConfigFlag(parser)
     flags.AddEnableIntraNodeVisibilityFlag(group)
-    flags.AddPeeringRouteSharingFlag(group)
-    flags.AddWorkloadIdentityFlags(group)
+    flags.AddWorkloadIdentityFlags(group, use_workload_pool=False)
     flags.AddWorkloadIdentityUpdateFlags(group)
-    flags.AddEnableShieldedNodesFlags(group)
     flags.AddDisableDefaultSnatFlag(group, for_cluster_create=False)
     flags.AddDatabaseEncryptionFlag(group)
     flags.AddDisableDatabaseEncryptionFlag(group)
     flags.AddCostManagementConfigFlag(group, is_update=True)
-    flags.AddReleaseChannelFlag(group, is_update=True)
+    flags.AddReleaseChannelFlag(group, is_update=True, hidden=False)
+    flags.AddEnableShieldedNodesFlags(group)
+    flags.AddTpuFlags(group, enable_tpu_service_networking=True)
+    flags.AddMasterGlobalAccessFlag(group)
+    flags.AddEnableGvnicFlag(group)
+    flags.AddNotificationConfigFlag(group, hidden=True)
+    flags.AddPrivateIpv6GoogleAccessTypeFlag('v1alpha1', group, hidden=True)
 
   def ParseUpdateOptions(self, args, locations):
+    flags.ValidateNotificationConfigFlag(args)
     opts = container_command_util.ParseUpdateOptionsBase(args, locations)
     opts.autoscaling_profile = args.autoscaling_profile
     opts.enable_pod_security_policy = args.enable_pod_security_policy
@@ -632,20 +724,50 @@ class UpdateAlpha(Update):
         args.clear_resource_usage_bigquery_dataset
     opts.security_profile = args.security_profile
     opts.istio_config = args.istio_config
+    opts.cloud_run_config = args.cloud_run_config
     opts.enable_intra_node_visibility = args.enable_intra_node_visibility
     opts.enable_network_egress_metering = args.enable_network_egress_metering
     opts.enable_resource_consumption_metering = args.enable_resource_consumption_metering
     flags.ValidateIstioConfigUpdateArgs(args.istio_config, args.disable_addons)
-    opts.enable_peering_route_sharing = args.enable_peering_route_sharing
+    flags.ValidateCloudRunConfigUpdateArgs(args.cloud_run_config, args.disable_addons)
+    if args.disable_addons and api_adapter.NODELOCALDNS in args.disable_addons:
+      # NodeLocalDNS is being enabled or disabled
+      console_io.PromptContinue(
+          message='Enabling/Disabling NodeLocal DNSCache causes a re-creation '
+          'of all cluster nodes at versions 1.15 or above. '
+          'This operation is long-running and will block other '
+          'operations on the cluster (including delete) until it has run '
+          'to completion.',
+          cancel_on_no=True)
+    if args.disable_addons and api_adapter.GCEPDCSIDRIVER in args.disable_addons:
+      pdcsi_disabled = args.disable_addons[api_adapter.GCEPDCSIDRIVER]
+      if pdcsi_disabled:
+        # GCE Persistent Disk CSI Driver is being disabled
+        console_io.PromptContinue(
+            message='If the GCE Persistent Disk CSI Driver is disabled, then any '
+            'pods currently using PersistentVolumes owned by the driver '
+            'will fail to terminate. Any new pods that try to use those '
+            'PersistentVolumes will also fail to start.',
+            cancel_on_no=True)
     opts.enable_stackdriver_kubernetes = args.enable_stackdriver_kubernetes
+    opts.enable_logging_monitoring_system_only = args.enable_logging_monitoring_system_only
+    opts.no_master_logs = args.no_master_logs
+    opts.master_logs = args.master_logs
+    opts.enable_master_metrics = args.enable_master_metrics
     opts.release_channel = args.release_channel
+    opts.enable_tpu = args.enable_tpu
+    opts.tpu_ipv4_cidr = args.tpu_ipv4_cidr
+    opts.enable_tpu_service_networking = args.enable_tpu_service_networking
 
     # Top-level update options are automatically forced to be
     # mutually-exclusive, so we don't need special handling for these two.
     opts.identity_namespace = args.identity_namespace
-    opts.disable_workload_identity = args.disable_workload_identity
     opts.enable_shielded_nodes = args.enable_shielded_nodes
     opts.disable_default_snat = args.disable_default_snat
     opts.enable_cost_management = args.enable_cost_management
+    opts.enable_master_global_access = args.enable_master_global_access
+    opts.enable_gvnic = args.enable_gvnic
+    opts.notification_config = args.notification_config
+    opts.private_ipv6_google_access_type = args.private_ipv6_google_access_type
 
     return opts
