@@ -19,10 +19,12 @@ from __future__ import division
 from __future__ import print_function
 from __future__ import unicode_literals
 
+from googlecloudsdk.api_lib.run import traffic
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.run import connection_context
 from googlecloudsdk.command_lib.run import exceptions
 from googlecloudsdk.command_lib.run import flags
+from googlecloudsdk.command_lib.run import messages_util
 from googlecloudsdk.command_lib.run import pretty_print
 from googlecloudsdk.command_lib.run import resource_args
 from googlecloudsdk.command_lib.run import serverless_operations
@@ -32,16 +34,17 @@ from googlecloudsdk.command_lib.util.concepts import presentation_specs
 from googlecloudsdk.core.console import progress_tracker
 
 
-@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
+@base.ReleaseTracks(base.ReleaseTrack.GA)
 class Update(base.Command):
-  """Update Cloud Run environment variables and other configuration settings.
-  """
+  """Update Cloud Run environment variables and other configuration settings."""
 
   detailed_help = {
-      'DESCRIPTION': """\
+      'DESCRIPTION':
+          """\
           {description}
           """,
-      'EXAMPLES': """\
+      'EXAMPLES':
+          """\
           To update one or more env vars:
 
               $ {command} myservice --update-env-vars=KEY1=VALUE1,KEY2=VALUE2
@@ -52,13 +55,16 @@ class Update(base.Command):
   def CommonArgs(parser):
     # Flags specific to managed CR
     managed_group = flags.GetManagedArgGroup(parser)
-    flags.AddServiceAccountFlag(managed_group)
     flags.AddCloudSQLFlags(managed_group)
+    flags.AddRevisionSuffixArg(managed_group)
+    flags.AddVpcConnectorArg(managed_group)
 
     # Flags specific to connecting to a cluster
     cluster_group = flags.GetClusterArgGroup(parser)
     flags.AddEndpointVisibilityEnum(cluster_group)
-    flags.AddCpuFlag(cluster_group)
+    flags.AddSecretsFlags(cluster_group)
+    flags.AddConfigMapsFlags(cluster_group)
+    flags.AddHttp2Flag(cluster_group)
 
     # Flags not specific to any platform
     service_presentation = presentation_specs.ResourcePresentationSpec(
@@ -74,12 +80,22 @@ class Update(base.Command):
     flags.AddAsyncFlag(parser)
     flags.AddLabelsFlags(parser)
     flags.AddMaxInstancesFlag(parser)
+    flags.AddCommandFlag(parser)
+    flags.AddArgsFlag(parser)
+    flags.AddPortFlag(parser)
+    flags.AddCpuFlag(parser)
+    flags.AddNoTrafficFlag(parser)
+    flags.AddServiceAccountFlag(parser)
     concept_parsers.ConceptParser([service_presentation]).AddToParser(parser)
 
   @staticmethod
   def Args(parser):
     Update.CommonArgs(parser)
 
+    # Flags only supported on GKE and Knative
+    cluster_group = flags.GetClusterArgGroup(parser)
+    flags.AddMinInstancesFlag(cluster_group)
+
   def Run(self, args):
     """Update configuration information about the service.
 
@@ -96,43 +112,48 @@ class Update(base.Command):
           '`--memory`, `--concurrency`, `--timeout`, `--connectivity`?')
 
     conn_context = connection_context.GetConnectionContext(
-        args, product=connection_context.Product.RUN)
+        args, flags.Product.RUN, self.ReleaseTrack())
     service_ref = flags.GetService(args)
 
     with serverless_operations.Connect(conn_context) as client:
-      deployment_stages = stages.ServiceStages()
+      service = client.GetService(service_ref)
+      has_latest = (
+          service is None or
+          traffic.LATEST_REVISION_KEY in service.spec_traffic)
+      deployment_stages = stages.ServiceStages(
+          include_iam_policy_set=False, include_route=has_latest)
       with progress_tracker.StagedProgressTracker(
           'Deploying...',
           deployment_stages,
           failure_message='Deployment failed',
           suppress_output=args.async_) as tracker:
-        client.ReleaseService(service_ref, changes, tracker, asyn=args.async_)
+        client.ReleaseService(
+            service_ref, changes, tracker, asyn=args.async_, prefetch=service)
       if args.async_:
-        pretty_print.Success(
-            'Deploying asynchronously.')
+        pretty_print.Success('Deploying asynchronously.')
       else:
-        service = client.GetService(service_ref)
-        active_revs = client.GetActiveRevisions(service_ref)
+        pretty_print.Success(
+            messages_util.GetSuccessMessageForSynchronousDeploy(
+                client, service_ref))
 
-        msg = ('Service [{{bold}}{serv}{{reset}}] revision{plural} {rev_msg} '
-               'is active and serving traffic at {{bold}}{url}{{reset}}')
 
-        rev_msg = ' '.join(
-            ['[{{bold}}{}{{reset}}]'.format(rev) for rev in active_revs])
+@base.ReleaseTracks(base.ReleaseTrack.BETA)
+class BetaUpdate(Update):
+  """Update Cloud Run environment variables and other configuration settings."""
 
-        msg = msg.format(
-            serv=service_ref.servicesId,
-            plural='s' if len(active_revs) > 1 else '',
-            rev_msg=rev_msg,
-            url=service.domain if 'domain' in dir(service) else service.url)
+  @staticmethod
+  def Args(parser):
+    Update.CommonArgs(parser)
+    flags.AddDeployTagFlag(parser)
 
-        pretty_print.Success(msg)
+    # Flags only supported on GKE and Knative
+    cluster_group = flags.GetClusterArgGroup(parser)
+    flags.AddMinInstancesFlag(cluster_group)
 
 
 @base.ReleaseTracks(base.ReleaseTrack.ALPHA)
 class AlphaUpdate(Update):
-  """Update Cloud Run environment variables and other configuration settings.
-  """
+  """Update Cloud Run environment variables and other configuration settings."""
 
   @staticmethod
   def Args(parser):
@@ -140,19 +161,11 @@ class AlphaUpdate(Update):
 
     # Flags specific to managed CR
     managed_group = flags.GetManagedArgGroup(parser)
-    flags.AddVpcConnectorArg(managed_group)
-    flags.AddRevisionSuffixArg(managed_group)
-
-    # Flags specific to connecting to a cluster
-    cluster_group = flags.GetClusterArgGroup(parser)
-    flags.AddSecretsFlags(cluster_group)
-    flags.AddConfigMapsFlags(cluster_group)
-    flags.AddHttp2Flag(cluster_group)
+    flags.AddEgressSettingsFlag(managed_group)
 
     # Flags not specific to any platform
     flags.AddMinInstancesFlag(parser)
-    flags.AddCommandFlag(parser)
-    flags.AddArgsFlag(parser)
-    flags.AddPortFlag(parser)
+    flags.AddDeployTagFlag(parser)
+
 
 AlphaUpdate.__doc__ = Update.__doc__
