@@ -41,7 +41,7 @@ from gslib.utils.constants import NO_MAX
 from gslib.utils.retry_util import Retry
 
 _SET_SYNOPSIS = """
-  gsutil acl set [-f] [-r] [-a] file-or-canned_acl_name url...
+  gsutil acl set [-f] [-r] [-a] <file-or-canned_acl_name> url...
 """
 
 _GET_SYNOPSIS = """
@@ -56,7 +56,7 @@ _CH_SYNOPSIS = """
     -u <id|email>:<perm>
     -g <id|email|domain|All|AllAuth>:<perm>
     -p <viewers|editors|owners>-<project number>:<perm>
-    -d <id|email|domain|All|AllAuth|<viewers|editors|owners>-<project number>>:<perm>
+    -d <id|email|domain|All|AllAuth|<viewers|editors|owners>-<project number>:<perm>
 """
 
 _GET_DESCRIPTION = """
@@ -68,8 +68,9 @@ _GET_DESCRIPTION = """
 _SET_DESCRIPTION = """
 <B>SET</B>
   The "acl set" command allows you to set an Access Control List on one or
-  more buckets and objects. The simplest way to use it is to specify one of
-  the canned ACLs, e.g.,:
+  more buckets and objects. The file-or-canned_acl_name parameter names either
+  a canned ACL or the path to a file that contains ACL text. The simplest way
+  to use the "acl set" command is to specify one of the canned ACLs, e.g.,:
 
     gsutil acl set private gs://bucket
 
@@ -92,16 +93,16 @@ _SET_DESCRIPTION = """
 
     gsutil acl set acl.txt gs://cats/file.txt
 
-  Note that you can set an ACL on multiple buckets or objects at once,
-  for example:
+  Note that you can set an ACL on multiple buckets or objects at once. For
+  example, to set ACLs on all .jpg files found in a bucket:
 
-    gsutil acl set acl.txt gs://bucket/*.jpg
+    gsutil acl set acl.txt gs://bucket/**.jpg
 
   If you have a large number of ACLs to update you might want to use the
   gsutil -m option, to perform a parallel (multi-threaded/multi-processing)
   update:
 
-    gsutil -m acl set acl.txt gs://bucket/*.jpg
+    gsutil -m acl set acl.txt gs://bucket/**.jpg
 
   Note that multi-threading/multi-processing is only done when the named URLs
   refer to objects, which happens either if you name specific objects or
@@ -112,16 +113,16 @@ _SET_DESCRIPTION = """
 <B>SET OPTIONS</B>
   The "set" sub-command has the following options
 
-    -R, -r      Performs "acl set" request recursively, to all objects under
-                the specified URL.
+  -R, -r      Performs "acl set" request recursively, to all objects under
+              the specified URL.
 
-    -a          Performs "acl set" request on all object versions.
+  -a          Performs "acl set" request on all object versions.
 
-    -f          Normally gsutil stops at the first error. The -f option causes
-                it to continue when it encounters errors. If some of the ACLs
-                couldn't be set, gsutil's exit status will be non-zero even if
-                this flag is set. This option is implicitly set when running
-                "gsutil -m acl...".
+  -f          Normally gsutil stops at the first error. The -f option causes
+              it to continue when it encounters errors. If some of the ACLs
+              couldn't be set, gsutil's exit status will be non-zero even if
+              this flag is set. This option is implicitly set when running
+              "gsutil -m acl...".
 """
 
 _CH_DESCRIPTION = """
@@ -150,8 +151,9 @@ _CH_DESCRIPTION = """
   Cache-Control header of "Cache-Control:private, max-age=0, no-transform" on
   such objects. For help doing this, see "gsutil help setmeta".
 
-  Grant anyone on the internet WRITE access to the bucket example-bucket
-  (WARNING: this is not recommended as you will be responsible for the content):
+  Grant anyone on the internet WRITE access to the bucket example-bucket:
+  
+  WARNING: this is not recommended as you will be responsible for the content
 
     gsutil acl ch -u AllUsers:W gs://example-bucket
 
@@ -161,9 +163,9 @@ _CH_DESCRIPTION = """
     gsutil acl ch -u john.doe@example.com:WRITE gs://example-bucket
 
   Grant the group admins@example.com OWNER access to all jpg files in
-  the top level of example-bucket:
+  example-bucket:
 
-    gsutil acl ch -g admins@example.com:O gs://example-bucket/*.jpg
+    gsutil acl ch -g admins@example.com:O gs://example-bucket/**.jpg
 
   Grant the owners of project example-project WRITE access to the bucket
   example-bucket:
@@ -260,21 +262,21 @@ _CH_DESCRIPTION = """
 <B>CH OPTIONS</B>
   The "ch" sub-command has the following options
 
-    -d          Remove all roles associated with the matching entity.
+  -d          Remove all roles associated with the matching entity.
 
-    -f          Normally gsutil stops at the first error. The -f option causes
-                it to continue when it encounters errors. With this option the
-                gsutil exit status will be 0 even if some ACLs couldn't be
-                changed.
+  -f          Normally gsutil stops at the first error. The -f option causes
+              it to continue when it encounters errors. With this option the
+              gsutil exit status will be 0 even if some ACLs couldn't be
+              changed.
 
-    -g          Add or modify a group entity's role.
+  -g          Add or modify a group entity's role.
 
-    -p          Add or modify a project viewers/editors/owners role.
+  -p          Add or modify a project viewers/editors/owners role.
 
-    -R, -r      Performs acl ch request recursively, to all objects under the
-                specified URL.
+  -R, -r      Performs acl ch request recursively, to all objects under the
+              specified URL.
 
-    -u          Add or modify a user entity's role.
+  -u          Add or modify a user entity's role.
 """
 
 _SYNOPSIS = (_SET_SYNOPSIS + _GET_SYNOPSIS.lstrip('\n') +
