@@ -86,8 +86,8 @@ _DETAILED_HELP_TEXT = ("""
     gsutil rewrite -k -r gs://bucket/subdir
 
   The rewrite command acts only on live object versions, so specifying a
-  URL with a generation will fail. If you want to rewrite an archived
-  generation, first copy it to the live version, then rewrite it, for example:
+  URL with a generation number fails. If you want to rewrite a noncurrent
+  version, first copy it to the live version, then rewrite it, for example:
 
     gsutil cp gs://bucket/object#123 gs://bucket/object
     gsutil rewrite -k gs://bucket/object
@@ -172,9 +172,12 @@ _DETAILED_HELP_TEXT = ("""
                 or CMEK). See 'gsutil help encryption' for details on encryption
                 configuration.
 
-  -O            Rewrite objects with the bucket's default object ACL instead of
-                the existing object ACL. This is needed if you do not have
-                OWNER permission on the object.
+  -O            When a bucket has uniform bucket-level access (UBLA) enabled,
+                the -O flag is required and will skip all ACL checks. When a
+                bucket has UBLA disabled, the -O flag rewrites objects with the
+                bucket's default object ACL instead of the existing object ACL.
+                This is needed if you do not have OWNER permission on the
+                object.
 
   -R, -r        The -R and -r options are synonymous. Causes bucket or bucket
                 subdirectory contents to be rewritten recursively.
@@ -389,9 +392,11 @@ class RewriteCommand(Command):
       src_metadata.acl = []
     elif not src_metadata.acl:
       raise CommandException(
-          'No OWNER permission found for object %s. OWNER permission is '
-          'required for rewriting objects, (otherwise their ACLs would be '
-          'reset).' % transform_url)
+          'No OWNER permission found for object %s. If your bucket has uniform '
+          'bucket-level access (UBLA) enabled, include the -O option in your '
+          'command to avoid this error. If your bucket does not use UBLA, you '
+          'can use the -O option to apply the bucket\'s default object ACL '
+          'when rewriting.' % transform_url)
 
     # Note: If other transform types are added, they must ensure that the
     # encryption key configuration matches the boto configuration, because
