@@ -329,7 +329,7 @@
 #define HAVE_SIGNAL_H 1
 /* #undef HAVE_SYS_DEVPOLL_H */
 /* #undef HAVE_SYS_EPOLL_H */
-/* #undef HAVE_SYS_EVENT_H */
+#define HAVE_SYS_EVENT_H 1
 #define HAVE_SYS_QUEUE_H 1
 #define HAVE_TAILQFOREACH 1
 #define USE_MB 1
@@ -497,7 +497,7 @@
 
 #define STACK_DIRECTION -1
 
-#define SYSTEM_TYPE "FreeBSD11.2"
+#define SYSTEM_TYPE "FreeBSD12.1"
 #define MACHINE_TYPE "amd64"
 /* #undef HAVE_DTRACE */
 
@@ -646,16 +646,16 @@
 
 #define MYSQL_VERSION_MAJOR 5
 #define MYSQL_VERSION_MINOR 6
-#define MYSQL_VERSION_PATCH 47
+#define MYSQL_VERSION_PATCH 50
 #define MYSQL_VERSION_EXTRA ""
 
 #define PACKAGE "mysql"
 #define PACKAGE_BUGREPORT ""
 #define PACKAGE_NAME "MySQL Server"
-#define PACKAGE_STRING "MySQL Server 5.6.47"
+#define PACKAGE_STRING "MySQL Server 5.6.50"
 #define PACKAGE_TARNAME "mysql"
-#define PACKAGE_VERSION "5.6.47"
-#define VERSION "5.6.47"
+#define PACKAGE_VERSION "5.6.50"
+#define VERSION "5.6.50"
 #define PROTOCOL_VERSION 10
 
 
