@@ -2,13 +2,13 @@
 from unittest import TestCase
 
 import pytest
-from lexicon.tests.providers.integration_tests import IntegrationTests
+from lexicon.tests.providers.integration_tests import IntegrationTestsV2
 
 
 # Hook into testing framework by inheriting unittest.TestCase and reuse
 # the tests which *each and every* implementation of the interface must
 # pass, by inheritance from define_tests.TheTests
-class Rage4ProviderTests(TestCase, IntegrationTests):
+class Rage4ProviderTests(TestCase, IntegrationTestsV2):
     """TestCase for Rage4"""
     provider_name = 'rage4'
     domain = 'capsulecd.com'
@@ -24,7 +24,7 @@ class Rage4ProviderTests(TestCase, IntegrationTests):
     def test_provider_when_calling_update_record_should_modify_record(self):
         return
 
-    # TODO: the following skipped suite and fixtures should be enabled
+    # TODO: enable the skipped tests
     @pytest.mark.skip(reason="new test, missing recording")
     def test_provider_when_calling_update_record_should_modify_record_name_specified(self):
         return
