@@ -1,13 +1,13 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2014-2018 Miroslav Stampar (@stamparm)
+Copyright (c) 2014-2020 Maltrail developers (https://github.com/stamparm/maltrail/)
 See the file 'LICENSE' for copying permission
 """
 
 from core.common import retrieve_content
 
-__url__ = "https://www.cruzit.com/xwbl2txt.php"
+__url__ = "https://www.cruzit.com/xxwbl2txt.php"
 __check__ = "ipaddress"
 __info__ = "known attacker"
 __reference__ = "cruzit.com"
