@@ -1,14 +1,14 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2014-2019 Maltrail developers (https://github.com/stamparm/maltrail/)
+Copyright (c) 2014-2020 Maltrail developers (https://github.com/stamparm/maltrail/)
 See the file 'LICENSE' for copying permission
 """
 
 from core.common import retrieve_content
 
 __url__ = "https://check.torproject.org/cgi-bin/TorBulkExitList.py?ip=1.1.1.1"
-__check__ = "Tor exit nodes"
+__check__ = "185.220."
 __info__ = "tor exit node (suspicious)"
 __reference__ = "torproject.org"
 
