@@ -33,7 +33,6 @@
 require_once("guiconfig.inc");
 require_once("interfaces.inc");
 require_once("filter.inc");
-require_once("services.inc");
 require_once("rrd.inc");
 require_once("system.inc");
 
@@ -71,7 +70,6 @@ function restore_config_section($section_name, $new_contents)
 
 $areas = array(
     'OPNsense' => gettext('OPNsense Additions'),	/* XXX need specifics */
-    'aliases' => gettext('Aliases'),
     'bridges' => gettext('Bridge Devices'),
     'ca' => gettext('SSL Certificate Authorities'),
     'cert' => gettext('SSL Certificates'),
@@ -190,6 +188,10 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         if (!empty($_POST['decrypt']) && empty($_POST['decrypt_password'])) {
             $input_errors[] = gettext('You must supply the password for decryption.');
         }
+        $user = getUserEntry($_SESSION['Username']);
+        if (userHasPrivilege($user, 'user-config-readonly')) {
+            $input_errors[] = gettext('You do not have the permission to perform this action.');
+        }
         /* read the file contents */
         if (is_uploaded_file($_FILES['conffile']['tmp_name'])) {
             $data = file_get_contents($_FILES['conffile']['tmp_name']);
@@ -284,10 +286,11 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
                     $filesInBackup = $provider['handle']->backup();
                 } catch (Exception $e) {
                     $filesInBackup = array();
+                    $input_errors[] = $e->getMessage();
                 }
 
                 if (count($filesInBackup) == 0) {
-                    $input_errors[] = gettext("communication failure");
+                    $input_errors[] = gettext('Saved settings, but remote backup failed.');
                 } else {
                     $input_messages = gettext("Backup successful, current file list:") . "<br>";
                     foreach ($filesInBackup as $filename) {
@@ -301,6 +304,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
 }
 
 include("head.inc");
+legacy_html_escape_form_data($pconfig);
 ?>
 
 <body>
@@ -367,7 +371,7 @@ $( document ).ready(function() {
                 </tr>
                 <tr>
                   <td>
-                    <input name="download" type="submit" class="btn btn-primary" value="<?=gettext("Download configuration"); ?>" />
+                    <input name="download" type="submit" class="btn btn-primary" value="<?= html_safe(gettext('Download configuration')) ?>" />
                   </td>
                 </tr>
                 <tr>
@@ -385,14 +389,16 @@ $( document ).ready(function() {
                 <tr>
                   <td>
                     <?=gettext("Restore area:"); ?>
-                    <select name="restorearea" id="restorearea" class="selectpicker">
-                      <option value=""><?=gettext("ALL");?></option>
+                    <div>
+                      <select name="restorearea" id="restorearea" class="selectpicker">
+                        <option value=""><?=gettext("ALL");?></option>
 <?php
-                    foreach($areas as $area => $areaname):?>
-                      <option value="<?=$area;?>"><?=$areaname;?></option>
+                      foreach($areas as $area => $areaname):?>
+                        <option value="<?=$area;?>"><?=$areaname;?></option>
 <?php
-                    endforeach;?>
-                    </select><br/>
+                      endforeach;?>
+                      </select>
+                    </div>
                     <input name="conffile" type="file" id="conffile" /><br/>
                     <input name="rebootafterrestore" type="checkbox" id="rebootafterrestore" checked="checked" />
                     <?=gettext("Reboot after a successful restore."); ?><br/>
@@ -410,7 +416,7 @@ $( document ).ready(function() {
                 </tr>
                 <tr>
                   <td>
-                    <input name="restore" type="submit" class="btn btn-primary" id="restore" value="<?=gettext("Restore configuration"); ?>" />
+                    <input name="restore" type="submit" class="btn btn-primary" id="restore" value="<?= html_safe(gettext('Restore configuration')) ?>" />
                   </td>
                 </tr>
                 <tr>
@@ -433,8 +439,12 @@ $( document ).ready(function() {
                     $fieldId = $providerId . "_" .$field['name'];?>
                     <tr>
                         <td style="width:22%">
-                            <a id="help_for_<?=$fieldId;?>" href="#" class="showhelp">
-                                <i class="fa fa-info-circle <?=empty($field['help']) ? "text-muted" : "";?>"></i></a> <?=$field['label'];?>
+<?php if (!empty($field['help'])): ?>
+                            <a id="help_for_<?=$fieldId;?>" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a>
+<?php else: ?>
+                            <i class="fa fa-info-circle text-muted"></i>
+<?php endif ?>
+                           <?=$field['label'];?>
                         </td>
                         <td style="width:78%">
 <?php
@@ -451,6 +461,9 @@ $( document ).ready(function() {
                         elseif ($field['type'] == 'password'):?>
 
                         <input name="<?=$fieldId;?>" type="password" value="<?=$field['value'];?>" />
+<?php
+                        elseif ($field['type'] == 'textarea'):?>
+                        <textarea name="<?=$fieldId;?>" rows="10"><?=$pconfig[$fieldId];?></textarea>
 <?php
                         endif;?>
                         <div class="hidden" data-for="help_for_<?=$fieldId;?>">
@@ -484,5 +497,5 @@ $( document ).ready(function() {
 include("foot.inc");
 
 if ($do_reboot) {
-    system_reboot();
+    configd_run('system reboot', true);
 }
