@@ -329,6 +329,7 @@ $main_buttons = array(
     array('label' => gettext('Add'), 'href' => 'firewall_rules_edit.php?if=' . $selected_if),
 );
 
+$a_filter_raw = config_read_array('filter', 'rule');
 legacy_html_escape_form_data($a_filter);
 $all_rule_stats = json_decode(configd_run("filter rule stats"), true);
 ?>
@@ -442,6 +443,7 @@ $( document ).ready(function() {
   $(".act_toggle").click(function(event){
       event.preventDefault();
       let target = $(this);
+      target.removeClass('fa-play').addClass('fa-spinner fa-pulse');
       let id = target.attr("id").split('_').pop(-1);
       $.ajax("firewall_rules.php",{
           type: 'post',
@@ -450,14 +452,18 @@ $( document ).ready(function() {
           data: {'act': 'toggle', 'id': id},
           success: function(response) {
               target.prop('title', response['new_label']).tooltip('fixTitle').tooltip('hide');
+              target.removeClass('fa-spinner fa-pulse').addClass('fa-play');
               if (response['new_state']) {
-                  target.find('span').removeClass('text-muted').addClass('text-success');
+                  target.removeClass('text-muted').addClass('text-success');
               } else {
-                  target.find('span').removeClass('text-success').addClass('text-muted');
+                  target.removeClass('text-success').addClass('text-muted');
               }
               $("#fw-alert-box").removeClass("hidden");
               $(".fw-alert-messages").addClass("hidden");
               $("#fw-alert-changes").removeClass("hidden");
+          },
+          error: function () {
+              target.removeClass('fa-spinner fa-pulse').addClass('fa-play');
           }
       });
   });
@@ -466,6 +472,7 @@ $( document ).ready(function() {
   $(".act_log").click(function(event){
       event.preventDefault();
       let target = $(this);
+      target.removeClass('fa-info-circle').addClass('fa-spinner fa-pulse');
       let id = target.attr("id").split('_').pop(-1);
       $.ajax("firewall_rules.php",{
           type: 'post',
@@ -474,14 +481,18 @@ $( document ).ready(function() {
           data: {'act': 'log', 'id': id},
           success: function(response) {
               target.prop('title', response['new_label']).tooltip('fixTitle').tooltip('hide');
+              target.removeClass('fa-spinner fa-pulse').addClass('fa-info-circle');
               if (response['new_state']) {
-                  target.find('i').removeClass('text-muted').addClass('text-info');
+                  target.removeClass('text-muted').addClass('text-info');
               } else {
-                  target.find('i').removeClass('text-info').addClass('text-muted');
+                  target.removeClass('text-info').addClass('text-muted');
               }
               $("#fw-alert-box").removeClass("hidden");
               $(".fw-alert-messages").addClass("hidden");
               $("#fw-alert-changes").removeClass("hidden");
+          },
+          error: function () {
+              target.removeClass('fa-spinner fa-pulse').addClass('fa-info-circle');
           }
       });
   });
@@ -504,7 +515,9 @@ $( document ).ready(function() {
       $(".rule").each(function(){
           if (selected_values.indexOf($(this).data('category')) == -1 && selected_values.length > 0) {
               $(this).hide();
+              $(this).find("input").prop('disabled', true);
           } else {
+              $(this).find("input").prop('disabled', false);
               $(this).show();
           }
       });
@@ -720,7 +733,7 @@ $( document ).ready(function() {
                 plugins_firewall($fw);
                 foreach ($fw->iterateFilterRules() as $rule):
                     $is_selected = $rule->getInterface() == $selected_if || (
-                        $rule->getInterface() == "" && $selected_if == "FloatingRules"
+                        ($rule->getInterface() == "" || strpos($rule->getInterface(), ",") !== false) && $selected_if == "FloatingRules"
                     );
                     if ($rule->isEnabled() && $is_selected):
                         $filterent = $rule->getRawRule();
@@ -778,20 +791,16 @@ $( document ).ready(function() {
                 ):
                   // calculate a hash so we can track these records in the ruleset, new style (mvc) code will
                   // automatically provide us with a uuid, this is a workaround to provide some help with tracking issues.
-                  $rule_hash = OPNsense\Firewall\Util::calcRuleHash($filterent);
+                  $rule_hash = OPNsense\Firewall\Util::calcRuleHash($a_filter_raw[$i]);
 ?>
                   <tr class="rule  <?=isset($filterent['disabled'])?"text-muted":"";?>" data-category="<?=!empty($filterent['category']) ? $filterent['category'] : "";?>">
                     <td>
                       <input class="rule_select" type="checkbox" name="rule[]" value="<?=$i;?>"  />
                     </td>
                     <td>
-                      <a href="#" class="act_toggle" id="toggle_<?=$i;?>" data-toggle="tooltip" title="<?=(empty($filterent['disabled'])) ? gettext("Disable Rule") : gettext("Enable Rule");?>">
-                        <span class="<?=firewall_rule_item_action($filterent);?>"></span>
-                      </a>
+                      <i class="act_toggle <?=firewall_rule_item_action($filterent);?>" style="cursor: pointer;" id="toggle_<?=$i;?>" data-toggle="tooltip" title="<?= html_safe(empty($filterent['disabled']) ? gettext('Disable rule') : gettext('Enable rule')) ?>"></i>
                       <?=firewall_rule_item_icons($filterent);?>
-                      <a href="#" class="act_log" id="toggle_<?=$i;?>" data-toggle="tooltip" title="<?=(empty($filterent['log'])) ? gettext("Enable Log") : gettext("Disable Log");?>">
-                        <i class="<?=firewall_rule_item_log($filterent);?>"></i>
-                      </a>
+                      <i class="act_log <?= firewall_rule_item_log($filterent) ?>" style="cursor: pointer;" id="toggle_<?=$i;?>" data-toggle="tooltip" title="<?= html_safe(empty($filterent['log']) ? gettext('Enable logging') : gettext('Disable logging')) ?>"></i>
                     </td>
                     <td class="view-info">
                         <?=firewall_rule_item_proto($filterent);?>
