@@ -30,19 +30,17 @@
 require_once("guiconfig.inc");
 require_once("system.inc");
 require_once("interfaces.inc");
-require_once("services.inc");
 
 $a_bridges = &config_read_array('bridges', 'bridged');
 
 // interface list
 $ifacelist = array();
-foreach (get_configured_interface_with_descr() as $bif => $bdescr) {
-    if (substr(get_real_interface($bif), 0, 3) != "gre") {
-        $ifacelist[$bif] = $bdescr;
+foreach (legacy_config_get_interfaces(array('virtual' => false, "enable" => true)) as $intf => $intfdata) {
+    if (substr($intfdata['if'], 0, 3) != 'gre' && substr($intfdata['if'], 0, 2) != 'lo') {
+        $ifacelist[$intf] = $intfdata['descr'];
     }
 }
 
-
 if ($_SERVER['REQUEST_METHOD'] === 'GET') {
     // read form data
     if (!empty($a_bridges[$_GET['id']])) {
@@ -57,8 +55,10 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             $pconfig[$fieldname] = null;
         }
     }
+
     // bool fields
-    $pconfig['enablestp'] = isset($a_bridges[$id]['enablestp']);
+    $pconfig['enablestp'] = !empty($a_bridges[$id]['enablestp']);
+    $pconfig['linklocal'] = !empty($a_bridges[$id]['linklocal']);
 
     // simple array fields
     $array_fields = array('members', 'stp', 'edge', 'autoedge', 'ptp', 'autoptp', 'static', 'private');
@@ -144,7 +144,12 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
 
     if (count($input_errors) == 0) {
         $bridge = array();
-        $bridge['enablestp'] = !empty($pconfig['enablestp']);
+        // booleans
+        foreach (['enablestp', 'linklocal'] as $fieldname) {
+            if (!empty($pconfig[$fieldname])) {
+                $bridge[$fieldname] = true;
+            }
+        }
         // 1 on 1 copy
         $copy_fields = array('descr', 'maxaddr', 'timeout', 'bridgeif', 'maxage','fwdelay', 'hellotime', 'priority', 'proto', 'holdcnt');
         foreach ($copy_fields as $fieldname) {
@@ -272,6 +277,18 @@ $(document).ready(function() {
                         </div>
                       </td>
                     </tr>
+                    <tr>
+                      <td><a id="help_for_linklocal" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?= gettext('Link-local address') ?></td>
+                      <td>
+                        <input type="checkbox" name="linklocal" <?= !empty($pconfig['linklocal']) ? 'checked="checked"' : '' ?> />
+                        <?= gettext('Enable link-local address') ?>
+                        <div class="hidden" data-for="help_for_linklocal">
+                          <?= gettext('By default, link-local addresses for bridges are disabled. You can enable them manually using this option. ' .
+                            'However, when a bridge interface has IPv6 addresses, IPv6 addresses on a member interface will be automatically ' .
+                            'removed before the interface is added.') ?>
+                        </div>
+                      </td>
+                    </tr>
                     <tr id="show_advanced_opt">
                       <td></td>
                       <td>
@@ -636,4 +653,7 @@ $(document).ready(function() {
       </div>
     </div>
   </section>
-<?php include("foot.inc"); ?>
+
+<?php
+
+include 'foot.inc';
