@@ -30,11 +30,9 @@
 
 require_once("guiconfig.inc");
 require_once("interfaces.inc");
-require_once("services.inc");
 
 $a_ppps = &config_read_array('ppps', 'ppp');
 
-
 if ($_SERVER['REQUEST_METHOD'] === 'GET') {
     // read form data
     $pconfig = array();
@@ -132,22 +130,22 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         $input_errors[] = gettext("The idle timeout value must be an integer.");
     }
 
-    foreach($pconfig['ports'] as $iface_idx => $iface){
+    foreach ($pconfig['ports'] as $iface_idx => $iface) {
         if (!empty($pconfig['localip'][$iface_idx]) && !is_ipaddr($pconfig['localip'][$iface_idx])) {
-            $input_errors[] = sprintf(gettext("A valid local IP address must be specified for %s."),$iface);
+            $input_errors[] = sprintf(gettext("A valid local IP address must be specified for %s."), $iface);
         }
-        if (!empty($pconfig['gateway'][$iface_idx]) && !is_ipaddr($pconfig['gateway'][$iface_idx]) && !is_hostname($pconfig['gateway'][$iface_idx])) {
-            $input_errors[] = sprintf(gettext("A valid gateway IP address OR hostname must be specified for %s."),$iface);
+        if (!empty($pconfig['gateway'][$iface_idx]) && !is_ipaddr($pconfig['gateway'][$iface_idx])) {
+            $input_errors[] = sprintf(gettext("A valid gateway IP address must be specified for %s."), $iface);
         }
         if (!empty($pconfig['bandwidth'][$iface_idx]) && !is_numericint($pconfig['bandwidth'][$iface_idx])) {
-            $input_errors[] = sprintf(gettext("The bandwidth value for %s must be an integer."),$iface);
+            $input_errors[] = sprintf(gettext("The bandwidth value for %s must be an integer."), $iface);
         }
 
         if (!empty($pconfig['mtu'][$iface_idx]) && $pconfig['mtu'][$iface_idx] < 576) {
-            $input_errors[] = sprintf(gettext("The MTU for %s must be greater than 576 bytes."),$iface);
+            $input_errors[] = sprintf(gettext("The MTU for %s must be greater than 576 bytes."), $iface);
         }
         if (!empty($pconfig['mru'][$iface_idx]) && $pconfig['mru'][$iface_idx] < 576) {
-            $input_errors[] = sprintf(gettext("The MRU for %s must be greater than 576 bytes."),$iface);
+            $input_errors[] = sprintf(gettext("The MRU for %s must be greater than 576 bytes."), $iface);
         }
     }
 
@@ -156,7 +154,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         $ppp['ptpid'] = $pconfig['ptpid'];
         $ppp['type'] = $pconfig['type'];
         $ppp['if'] = $ppp['type'].$ppp['ptpid'];
-        $ppp['ports'] = implode(',',$pconfig['ports']);
+        $ppp['ports'] = implode(',', $pconfig['ports']);
         $ppp['username'] = $pconfig['username'];
         $ppp['password'] = base64_encode($pconfig['password']);
         $ppp['ondemand'] = !empty($pconfig['ondemand']);
@@ -171,8 +169,8 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         // Loop through fields associated with a individual link/port and make an array of the data
         $port_fields = array("localip", "gateway", "subnet", "bandwidth", "mtu", "mru", "mrru");
         $port_data = array();
-        foreach($pconfig['ports'] as $iface_idx => $iface){
-            foreach($port_fields as $field_label){
+        foreach ($pconfig['ports'] as $iface_idx => $iface) {
+            foreach ($port_fields as $field_label) {
                 if (!isset($port_data[$field_label])) {
                     $port_data[$field_label] = array();
                 }
@@ -190,13 +188,13 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
                   $ppp['simpin'] = $pconfig['simpin'];
                   $ppp['pin-wait'] = $pconfig['pin-wait'];
                 }
-                if (!empty($pconfig['apn'])){
+                if (!empty($pconfig['apn'])) {
                   $ppp['apn'] = $pconfig['apn'];
                   $ppp['apnum'] = $pconfig['apnum'];
                 }
                 $ppp['phone'] = $pconfig['phone'];
-                $ppp['localip'] = implode(',',$port_data['localip']);
-                $ppp['gateway'] = implode(',',$port_data['gateway']);
+                $ppp['localip'] = implode(',', $port_data['localip']);
+                $ppp['gateway'] = implode(',', $port_data['gateway']);
                 if (!empty($pconfig['connect-timeout'])) {
                     $ppp['connect-timeout'] = $pconfig['connect-timeout'];
                 }
@@ -213,9 +211,9 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
                 break;
             case "pptp":
             case "l2tp":
-                $ppp['localip'] = implode(',',$port_data['localip']);
-                $ppp['subnet'] = implode(',',$port_data['subnet']);
-                $ppp['gateway'] = implode(',',$port_data['gateway']);
+                $ppp['localip'] = implode(',', $port_data['localip']);
+                $ppp['subnet'] = implode(',', $port_data['subnet']);
+                $ppp['gateway'] = implode(',', $port_data['gateway']);
                 break;
             default:
                 break;
@@ -258,9 +256,9 @@ include("head.inc");
 
 <body>
   <script>
-    $(document).ready(function() {
+    $(document).ready(function () {
         // change type
-        $("#type").change(function(){
+        $("#type").change(function () {
           $('#ppp,#ppp_adv,#pppoe,#hostuniqopt,#ppp_provider,#phone_num,#apn_').hide();
           $('#ports > [data-type="serial"]').hide();
           $('#ports > [data-type="serial"]').prop('disabled', true);
@@ -302,7 +300,7 @@ include("head.inc");
         $("#type").change();
 
         // change interfaces / ports selection
-        $("#ports").change(function(){
+        $("#ports").change(function () {
             for (i=0; i <= $("#ports").children().length; ++i) {
                 if ($('#ports :selected').length > i) {
                     $(".intf_select_"+i).prop('disabled', false);
@@ -314,7 +312,7 @@ include("head.inc");
             }
             // add item text
             var i=0;
-            $('#ports :selected').each(function(){
+            $('#ports :selected').each(function () {
               $(".intf_select_txt_"+i).html('( '+ $(this).val() + ' )');
               i++;
             });
@@ -322,13 +320,13 @@ include("head.inc");
         $("#ports").change();
 
         // advanced options
-        $("#show_advanced").click(function(){
+        $("#show_advanced").click(function () {
             $(".act_show_advanced").show();
             $("#show_advanced_opt").hide();
         });
 
         // ppp -> country change
-        $("#country").change(function(){
+        $("#country").change(function () {
             $('#provider_list').children().remove();
             $('#providerplan').children().remove();
             $.ajax("getserviceproviders.php",{
@@ -557,7 +555,7 @@ include("head.inc");
                   <table class="table table-striped" id="interface_details" style="display:none">
                     <tbody>
 <?php
-                      for ($intf_idx=0; $intf_idx <= max(count($serialports), count($portlist)) ; ++$intf_idx):?>
+                      for ($intf_idx=0; $intf_idx <= count($portlist) ; ++$intf_idx):?>
                       <tr style="display:none" class="intf_select_<?=$intf_idx;?>">
                         <td style="width:22%"><a id="help_for_localip_<?=$intf_idx;?>" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("Local IP");?> <span class="intf_select_txt_<?=$intf_idx;?>"> </span></td>
                         <td style="width:78%">
@@ -580,7 +578,7 @@ include("head.inc");
                         <td style="width:78%">
                           <input name="gateway[]" type="text" class="intf_select_<?=$intf_idx;?>" value="<?=isset($pconfig['gateway'][$intf_idx]) ? $pconfig['gateway'][$intf_idx] : "";?>" />
                           <div class="hidden" data-for="help_for_gateway_<?=$intf_idx;?>">
-                            <?= gettext("IP Address OR Hostname"); ?>
+                            <?= gettext("IP Address"); ?>
                           </div>
                         </td>
                       </tr>
@@ -758,7 +756,7 @@ include("head.inc");
                   <table class="table table-striped act_show_advanced" style="display:none">
                     <tbody>
 <?php
-                      for ($intf_idx=0; $intf_idx <= max(count($serialports), count($portlist)) ; ++$intf_idx):?>
+                      for ($intf_idx=0; $intf_idx <= count($portlist); ++$intf_idx):?>
                       <tr style="display:none" class="intf_select_<?=$intf_idx;?>">
                         <td style="width:22%"> <a id="help_for_link_<?=$intf_idx;?>" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a>  <?=gettext("Link Parameters");?> <span class="intf_select_txt_<?=$intf_idx;?>"> </span></td>
                         <td style="width:78%">
