@@ -30,7 +30,6 @@
 
 require_once("guiconfig.inc");
 require_once("config.inc");
-require_once("services.inc");
 require_once("interfaces.inc");
 require_once("plugins.inc.d/dhcpd.inc");
 
@@ -258,9 +257,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
     );
 } elseif ($_SERVER['REQUEST_METHOD'] === 'POST') {
     if (!empty($_POST['deleteip']) && is_ipaddr($_POST['deleteip'])) {
-        // delete dhcp lease
-        /* Stop DHCPD */
-        killbyname("dhcpd");
+        killbypid('/var/dhcpd/var/run/dhcpd.pid', 'TERM', true);
         $fin = @fopen($leasesfile, "r");
         $fout = @fopen($leasesfile.".new", "w");
         if ($fin) {
@@ -424,7 +421,9 @@ legacy_html_escape_form_data($leases);
                   <td><?=$data['descr'];?></td>
                   <td><?= !empty($data['start']) ? adjust_utc($data['start']) : '' ?></td>
                   <td><?= !empty($data['end']) ? adjust_utc($data['end']) : '' ?></td>
-                  <td><?=$data['online'];?></td>
+                  <td>
+                    <i class="fa fa-<?=$data['online']=='online' ? 'signal' : 'ban';?>" title="<?=$data['online'];?>" data-toggle="tooltip"></i>
+                  </td>
                   <td><?=$data['act'];?></td>
                   <td class="text-nowrap">
 <?php if (!empty($data['if'])): ?>
